/*
 * Copyright (c) 1990-2001 by George White Consulting
 *          All Rights Reserved
 * 
 * Anyone who uses this code does so at their own risk. No
 * warranty of any kind is expressed or implied.  Any
 * commercial use of this code is prohibited unless agreed
 * to in writing.
 */
/*
 *  ftest f1 f2 ... fN [-options]
 *
 *  This program returns 1 if argc > 1 and the file f1
 *	exists and is older by a specific amount than files f2 - fN
 *  It returns 0 otherwise
 *  These seemingly backwards return values are for sh if commands
 *
 *  options
 *	-day N	    N days older    (default 1 day)
 *	-hour N	    N hours older   (default 1 hour)
 *	-min N	    N minutes older (default 1 min)
 *	-sec N	    N seconds older (default 1 sec)
 *	default 0 seconds
 *
 *  -n option sets the date of f1 to the specified amount of
 *	time later than the newest of f2 - fN
 *  (same as fupdate f1 f2 ... fN)
 *
 *  -o can set the date of f1 to 1 minute older than the
 *	oldest of f2 - fN
 *  same as fupdate - f1 f2 ... fN
 *
 *  for -n and -o options, time defaults to 60 seconds
 *
 * -q acts silently   (default to test)
 * -d prints messages (default for update)
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/errno.h>

main(argc, argv)
int	argc;
char	**argv;
{
    struct stat	buf;
    char    *c, *cc;
    int	    i;
    int	    update;
    int	    max;
    int	    quiet;
    struct timeval tv[2];
    extern int errno;

    if(--argc < 2) exit(0);
    quiet = 1;
    update = 0;
    c = cc = *argv;
    while(*c) {
	if((c[0] == '/') && c[1]) cc = &c[1];
	c++;
    }
    if(!strcmp(cc, "fupdate")) {
	update = 1;
	quiet = 0;
    }
    argv++;

    i = 0;
    while(*argv[0] == '-') {
	if(!*argv[1]) update = -update;
	if(!strcmp(*argv, "-o")) {
	    update = -1;
	    quiet = 0;
	} else if(!strcmp(*argv, "-n")) {
	    update = 1;
	    quiet = 0;
	} else if(!strcmp(*argv, "-q")) {
	    quiet = 1;
	} else if(!strcmp(*argv, "-d")) {
	    quiet = 0;
	} else if(!strcmp(*argv, "-day")) {
	    argv++; argc--;
	    if(!argv) exit(0);
	    if(sscanf(*argv, "%d", &i) == 1) {
		i = i * 60 * 60 * 24;
	    } else i = 60 * 60 * 24;
	} else if(!strcmp(*argv, "-hour")) {
	    argv++; argc--;
	    if(!argv) exit(0);
	    if(sscanf(*argv, "%d", &i) == 1) {
		i = i * 60 * 60;
	    } else i = 60 * 60;
	} else  if(!strcmp(*argv, "-min")) {
	    argv++; argc--;
	    if(!argv) exit(0);
	    if(sscanf(*argv, "%d", &i) == 1) {
		i = i * 60;
	    } else i = 60;
	} else if(!strcmp(*argv, "-sec")) {
	    argv++; argc--;
	    if(!argv) exit(0);
	    if(sscanf(*argv, "%d", &i) != 1) i = 1;
	} else {
	    printf("ftest: unknown option '%s'\n", *argv);
	}
	argv++; argc--;
    }
    if(argc <= 1) exit(0);

    c = *argv;
    if(!update) {
	if(stat(c, &buf) < 0) {
	    if(!quiet) printf("ftest: file %s does not exist\n", c);
	    exit(0);
	}
	i += buf.st_mtime;
	while(--argc > 0) {
	    argv++;
	    if(stat(*argv, &buf) < 0) exit(0);
	    if(buf.st_mtime > i) exit(0);
	}
	if(!quiet) printf("file %s is up to date\n", c);
	exit(1);
    } else {
	if(!i) i = 60;
	if(stat(c, &buf) < 0) {
	    if(!quiet) printf("ftest: file %s does not exist\n", c);
	    exit(1);
	}
	if(update < 0)
	    max = 0x7fffffff;
	else
	    max = 0;
	tv[0].tv_sec = buf.st_atime;
	tv[0].tv_usec = 0;
	tv[1].tv_sec = buf.st_mtime;
	tv[1].tv_usec = 0;
	
	while(--argc > 0) {
	    argv++;
	    if(stat(*argv, &buf) < 0) {
		if(!quiet)
		    printf("ftest: stat returned errno = %d for %s", errno, *argv);
		continue;
	    }
	    if(update < 0) {
		if(buf.st_mtime < max) max = buf.st_mtime;
	    } else {
		if(buf.st_mtime > max) max = buf.st_mtime;
	    }
	}
	if((max > 0) && (max != 0x7fffffff)) {
	    if(update < 0)
		max -= i;		/* subtract a minute */
	    else
		max += i;		/* add a minute */
	    tv[1].tv_sec = max;
/*
 * ctime() includes \n
 */
	    if(!quiet)
		printf("set time of %s to %s", c, ctime((time_t *) &max));
	    if(utimes(c, tv))
		printf("ftest: utimes returned errno = %d", errno);
	}
	exit(0);
    }
}
