/*
 * Copyright (c) 1990-2001 by George White Consulting
 *          All Rights Reserved
 * 
 * Anyone who uses this code does so at their own risk. No
 * warranty of any kind is expressed or implied.  Any
 * commercial use of this code is prohibited unless agreed
 * to in writing.
 */
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <signal.h>
/*
 * this is a snoop logger and is expected to be reading directly from
 * snoop or from a fifo to which the output of snoop is sent
 *
 * snoop -d elxl0 | snoop_logger </usr/adm/snoop_log
 *
 * snoop -d elxl0 | tee /etc/admin/snoop_pipe &
 * snoop_logger <snoop_pipe >/usr/adm/snoop_log
 *
 * writes the date to the output when something is logged if the
 * date has not already been written in the last 30 minutes
 */

/*
 * packets that match these strings will be ignored
 */
#define MAXS	12
char *ignore_strings[MAXS] = {
" SMTP ",
" DNS ",
" HTTP ",
" HTTPS ",
" FTP",	    /* catch FTP and FTP-DATA */
" ARP ",
" NTP ",
" TELNET ",
" NICNAME ",
" TCP ",
" D=224.0.0.1",
" ALL-SYSTEMS.MCAST.NET ICMP"
};
/*
 * The first packet that matches these strings, per time period
 * will be logged and the others will be counted and reported
 */
#define MAXL	10
char *log_strings[MAXL] = {
" TELNET R",
" TELNET C",
" FTP R",
" FTP C",
" TCP*Syn ",
" TCP*Fin ",
" TCP ",
" XWIN R",
" XWIN C",
" ICMP Echo",
};

int log_stat[MAXL] = { 0 };

int dotime;

#define TIME_PERIOD 30
/* #define TIME_PERIOD 1 */

main(argc, argv)
int argc;
char	**argv;
{
    char    buffer[1024];
    char    *c, *cc, *tcc;
    int i, j, k, dolog;
    time_t  t;

    setalarm();

    i = getc(stdin);
    while(!feof(stdin)) {
	c = buffer;
	while(i != '\n') {
	    *c++ = i;
	    i = getc(stdin);
	}
	*c = 0;
	dolog = buffer[0] ? 1 : 0;
	for(i = 0; i < MAXS; i++) {
	    cc = ignore_strings[i];
	    c = buffer;
	    while(*c) {
		if(*c++ == *cc) {
		    cc++;
		    while(*cc && (*c == *cc)) {
			c++; cc++;
		    }
		    if(!*cc) dolog = 0;
		    cc = ignore_strings[i];
		}
	    }
	}
	for(i = 0; i < MAXL; i++) {
	    if(log_stat[i] > 256) continue;
	    tcc = cc = log_strings[i];
	    c = buffer;
	    j = 1;
	    k = 0;
	    while(*c) {
		if(*c++ == *cc) {
		    cc++;
		    while(*cc && (*c == *cc)) {
			c++; cc++;
			if(!j) j = 1;
			if(*cc == '*') {
			    j = 0;
			    cc++;
			    tcc = cc;
			}
		    }
		    if(!*cc && j) {
			if(!log_stat[i]) dolog = 1;
			log_stat[i] += 1;
			k = 1;
			break;
		    }
		    cc = tcc;
		}
	    }
	    if(k) break;
	}
	if(dotime) {
	    j = 0;
	    for(i = 0; i < MAXL; i++) {
		if(log_stat[i] > 1) j++;
	    }
	    if(j) {
		t = time(0);
		fprintf(stdout, "%s", ctime(&t));
		for(i = 0; i < MAXL; i++) {
		    j = log_stat[i];
		    if(j <= 1) continue;
		    fprintf(stdout, "COUNT '%s' = %d\n",
			    log_strings[i], j);
		    log_stat[i] = 0;
		}
		dotime = 0;
		reset_timer();
	    }
	}
	if(dolog) {
	    if(dotime) {
		t = time(0);
		fprintf(stdout, "%s", ctime(&t));
		dotime = 0;
		reset_timer();
	    }
	    fprintf(stdout, "%s\n", buffer);
	    fflush(stdout);
	}
	i = getc(stdin);
    }
    return(0);
}

setalarm()
{
    struct sigaction sigact;
    sigset_t	mask;
    static void doalarm();

    if(sigemptyset(&mask))
	fprintf(stdout, "1 errno = %d\n");

    sigact.sa_flags = SA_RESTART;
    sigact.sa_handler = doalarm;
    sigact.sa_mask = mask;

    if(sigaddset(&mask, SIGALRM))
	fprintf(stdout, "2 errno = %d\n");
    if(sigprocmask(SIG_UNBLOCK, &mask, (sigset_t *) 0))
	fprintf(stdout, "3 errno = %d\n");
    if(sigaction(SIGALRM, &sigact, (struct sigaction *) 0))
	fprintf(stdout, "4 errno = %d\n");

    reset_timer();
}

static doalarm()
{
    dotime = 1;
}

reset_timer()
{
    struct itimerval	it;

    it.it_interval.tv_sec = it.it_value.tv_sec = TIME_PERIOD * 60;
    it.it_interval.tv_usec = it.it_value.tv_usec = 0;
    if(setitimer(ITIMER_REAL, &it, 0))
	fprintf(stdout, "5 errno = %d\n");
}


